import setuptools

setuptools.setup(
    name="cwinput",
    version="0.1.2",
    author="Zhilkaidarov Damir",
    description="Listen to raw input from console",
    packages=[
        "cwinput",
    ],
    classifiers=[
        "Development Status :: 3 - Alpha"
    ]
)
